/************************************************************************ *********
 * Proiectarea cu Microprocesoare
 * Laboratorul 1
 * Digital I/O
 *********************************************************************************/

#include <avr\io.h>
#include <avr\interrupt.h>
#define F_CPU 16000000
#include <util/delay.h>

#include <string.h>
#include <stdio.h>

#include "lcd.h"

char s_state;
char en;
int  global 	= 0;
char bit;
char bit_count 	= 0;
char byte 		= 0;
char byte_count	= 12;
char str[16];

char r_state = 1;
char g_byte = 0;
char g_bit = 0;
char g_bit_count = 0;
char mes[16]={0};
char aux[10];
int val[16];


ISR(TIMER0_COMPA_vect) 
{
	if (en != 0){
		// invert state of PD5
		PORTD = ((1<< PD5) ^ PORTD);
	}
	
	if (s_state == 1)		// send initial logical '1' for 9 ms as start tag
	{
		if (global == 692)
		{
			s_state = 2;
			global = 0;
			en = 0;
		}
	}
	else if (s_state == 2)	// send initial logical '0' for 4.5 ms
	{
		if (global == 346)
		{
			s_state = 3;
			global = 0;
			en = 1;  
			bit = (str[0] & 0x01);
		}
	}
	else if (s_state == 3)		// sending bit of data state
	{
		if (en == 1){			// first oscilation part
			if ((bit == 0 && global == 43) ||
				(bit == 1 && global == 180)){
				en = 0;
				global = 0;
			}
		}
		else if (global == 43){		// second clear part
			if (byte == byte_count){
				// turn counter off
				TCCR0B &= ~(1 << CS00);
				PORTD &= ~(1 << PD5);
			}
			
			bit_count++;
			// jump to next byte if necessary
			if (bit_count == 8){
				bit_count = 0;
				byte++;
			}
		
			bit = (str[byte] & (1<<bit_count)) > 0;
			global = 0;
			en = 1;
		}
	}
	global++;
}

ISR(PCINT3_vect)
{/*
	if ((PIND & (1<<PD6)) == 0)	// send button has been pressed
	{
		PORTD |= (1<<PD7);
	}
	else
	{
		PORTD &= ~(1<<PD7);
	}
	*/
	cli();
	char i;
	for (i=0; i < 16; i++)
	{
		LCD_writeInstruction(LCD_INSTR_clearDisplay);
		sprintf(aux, "%d", val[i]);
		LCD_printn(aux, sizeof(aux));
		_delay_ms(5000);
	}
}

ISR(PCINT2_vect)
{

	PORTD |= (1 << PD7);
	
}

void send()
{
	en = 1;		// set oscilation on (send a 9ms long '1')
	s_state = 1;
	
	// use timer 0 to send carrier frequency
	TCCR0B |= (1 << CS00);		// start timer 0, no prescaling
}

/* Afisare mesaj simplu */
void init_transmission()
{	
	DDRD 	|= (1 << PD7);			// set PD7 as output
	
	DDRD 	&= ~(1 << PD6);  		// pin PD6 as input
	PORTD 	|= (1 << PD6);  		// activate pull-up
	
	DDRD 	|= (1 << PD5);			// set PD5 as output
	PORTD 	|= (1 << PD5);			// set PD5 on 1
	
	DDRC 	&= ~(1 << PC7);		// set PC7 as input
	PORTC 	|= (1 << PC7);			// activate pull up rezistor
	
	PCICR 	|= (1 << PCIE3);		// enable iterrupts on PCINT24:31 => PCIE3
	PCICR 	|= (1 << PCIE2);		// enable iterrupts on PCINT16:23 => PCIE2
	PCMSK3 	|= (1 << PCINT30);		// enable iterrupt for pin PD6 (INT30)
	PCMSK2 	|= (1 << PCINT23);		// enable iterrupt for pin PC7 (INT23)
	
	sei();		// enable global interupts
	
	// init the counter/timer1 (16 bit)
	//TIMSK1 |= (1 << OCIE1A);	// enable interrupt on OCR1A
	TCCR1A |= (1 << COM1B0);	// what to toggle on compare match
	TCCR1B |= (1 << WGM12);
	OCR1A 	= 20000;			// units to count
	
	// init timer 0 (8 bit)
	TIMSK0 |= (1 << OCIE0A);	// enable interrupt on OCR0A
	TCCR0A |= (1 << COM0B0);
	TCCR0B |= (1 << WGM02);	// toggle OC0A on match
	OCR0A 	= 208;				// units to count
}

int main()
{	
	
	init_transmission();
	LCD_init();		// init LCD
	
	_delay_ms(1000);
	
	
	
	
	while(1);
	
	return 0;
}
